<?php

/**
 * Extends the WC_Settings_Page class
 *
 * @link        https://paulmiller3000.com
 * @since       1.0.0
 *
 * @package     P3k_Galactica
 * @subpackage  P3k_Galactica/admin
 *
 */

if (!defined('ABSPATH')) exit; // Exit if accessed directly

if (!class_exists('Freeinvoice_Api_WC_Settings')) {

	/**
	 * Settings class
	 *
	 * @since 1.0.0
	 */
	class Freeinvoice_Api_WC_Settings extends WC_Settings_Page
	{

		/**
		 * Constructor
		 * @since  1.0
		 */
		public function __construct()
		{

			$this->id    = 'freeinvoice-api';
			$this->label = __('FreeInvoice API', 'freeinvoice-api');

			// Define all hooks instead of inheriting from parent                    
			add_filter('woocommerce_settings_tabs_array', array($this, 'add_settings_page'), 20);
			add_action('woocommerce_sections_' . $this->id, array($this, 'output_sections'));
			add_action('woocommerce_settings_' . $this->id, array($this, 'output'));
			add_action('woocommerce_settings_save_' . $this->id, array($this, 'save'));
		}

		/**
		 * Get sections.
		 *
		 * @return array
		 */
		public function get_sections()
		{
			$sections = array(
				'' => __('Settings'),
				'advanced_tax' => __('Gestione Aliquote IVA 0%', 'freeinvoice-api'),
				'payment_methods' => __('Gestione Metodi di pagamento', 'freeinvoice-api'),
			);
			if (empty(Freeinvoice_Api_Validator::config_errors()))
				$sections['info_invoice_pack'] = __('Info Lotti Fattura', 'freeinvoice-api');
			else
				$sections['buy_invoice_pack'] = __('Acquisto Lotti Fattura', 'freeinvoice-api');
			$sections['documentation'] = __('Documentation');

			return apply_filters('woocommerce_get_sections_' . $this->id, $sections);
		}
		/**
		 *	Output sections
		 */
		public function output_sections()
		{

			global $current_section;

			$sections = $this->get_sections();

			if (empty($sections) || 1 === sizeof($sections)) {
				return;
			}

			echo '<ul class="subsubsub">';

			$array_keys = array_keys($sections);

			foreach ($sections as $id => $label) {
				switch ($id) {
					case 'buy_invoice_pack':
						echo '<li><a href="' . Freeinvoice_Api_Comm::LINK_BUY_API . '" target="_blank" class="' . ($current_section == $id ? 'current' : '') . '">' . $label . '</a> ' . (end($array_keys) == $id ? '' : '|') . ' </li>';
						break;
					case 'documentation':
						echo '<li><a href="' . Freeinvoice_Api_Comm::LINK_DOCUMENTATION . '" target="_blank" class="' . ($current_section == $id ? 'current' : '') . '">' . $label . '</a> ' . (end($array_keys) == $id ? '' : '|') . ' </li>';
						break;
					default:
						echo '<li><a href="' . admin_url('admin.php?page=wc-settings&tab=' . $this->id . '&section=' . sanitize_title($id)) . '" class="' . ($current_section == $id ? 'current' : '') . '">' . $label . '</a> ' . (end($array_keys) == $id ? '' : '|') . ' </li>';
				}
			}

			echo '</ul><br class="clear" />';
		}

		/**
		 * Get settings array
		 *
		 * @return array
		 */
		public function get_settings()
		{

			global $current_section;
			$prefix = Freeinvoice_Api_Billing::DB_PREFIX;
			switch ($current_section) {
				case 'advanced_tax':
					include 'partials/wc-settings/freeinvoice-api-settings-advanced-tax.php';
					break;
				case 'payment_methods': //'<a href="' .admin_url('admin.php?page=wc-settings&tab=' . $this->id . '&section=' . sanitize_title('payment_methods')) . '">' . $label . '</a> '
					include 'partials/wc-settings/freeinvoice-api-settings-payment-methods.php';
					break;
				case 'info_invoice_pack':
					$settings = array();
					include 'partials/wc-settings/freeinvoice-api-settings-stocks.php';
					break;
				default:
					include 'partials/wc-settings/freeinvoice-api-settings-main.php';
			}

			return apply_filters('woocommerce_get_settings_' . $this->id, $settings, $current_section);
		}

		/**
		 * Output the settings
		 */
		public function output()
		{
			$settings = $this->get_settings();

			$err = Freeinvoice_Api_Validator::config_errors();
			$i = $msg = null;
			foreach ($err as $field_name => $error) {
				$i = array_search($field_name, array_column($settings, 'id'));
				if (isset($settings[$i]['class']))
					$settings[$i]['class'] .= ' freeinvoice-api-border-red';
				else
					$settings[$i]['class'] = 'freeinvoice-api-border-red';

				if (is_array($error)) {
					$msg = __('Errore nella verifica dell\'api key. Risposta dal server:', 'freeinvoice-api') . '&nbsp;';
					$multiple_errors = array();
					foreach ($error as $e)
						$multiple_errors[] = $e['message'] . " ({$e['code']})";
					$msg .= implode(' - ', $multiple_errors);
					// $msg .=json_encode($error);
				} else
					$msg = __($error, 'freeinvoice-api');
				echo '<div class="notice notice-error"><p>' . $msg . '</p></div>';
			}

			WC_Admin_Settings::output_fields($settings);
		}

		/**
		 * Save settings
		 *
		 * @since 1.0
		 */
		public function save()
		{
			// if ($response === true) {
			$settings = $this->get_settings();
			global $current_section;

			switch ($current_section) {
				case 'advanced_tax':
					Freeinvoice_Api_Billing::update_natura_taxes($_POST);
					break;
				case 'payment_methods':
					Freeinvoice_Api_Billing::update_payment_config($_POST);
					break;
				default:
					$this->validate_post_fields($_POST);
					Freeinvoice_Api_Validator::remove_setting_errors();
					WC_Admin_Settings::save_fields($settings);
			}
		}

		private function validate_post_fields(&$post)
		{
			$settings = array_map(function ($val) {
				return $val['id'];
			}, $this->get_settings());
			$prefix = Freeinvoice_Api_Billing::DB_PREFIX;
			foreach ($settings as $id) if (isset($post[$id])) {
				$value = &$post[$id];
				$value = trim($value);
				if ($id === $prefix . 'vatcode_field' || $id === $prefix . 'taxcode_field')
					$value = Freeinvoice_Api_Validator::validate_italian_vat_code($value);
			}
			$next_invoice = &$post[$prefix . 'next_invoice_field'];
			if (intval($next_invoice) <= 0)
				$next_invoice = 1;
		}
	}
}


return new Freeinvoice_Api_WC_Settings();
