(function($) {
    'use strict';

    /**
     * All of the code for your admin-facing JavaScript source
     * should reside in this file.
     *
     * Note: It has been assumed you will write jQuery code here, so the
     * $ function reference has been prepared for usage within the scope
     * of this function.
     *
     * This enables you to define handlers, for when the DOM is ready:
     *
     * $(function() {
     *
     * });
     *
     * When the window is loaded:
     *
     * $( window ).load(function() {
     *
     * });
     *
     * ...and/or other possibilities.
     *
     * Ideally, it is not considered best practise to attach more than a
     * single DOM-ready or window-load handler for a particular page.
     * Although scripts in the WordPress core, Plugins and Themes may be
     * practising this, we should strive to set a better example in our own work.
     */

    $(function() {
        var $dropdown = $('.cffi-dropdown'),
            $date_pickers = $('.freeinvoice-api-datepicker'),
            $sectional_controller = $('#_freeinvoice_api_sectional_enabled'),
            $invoice_number_controller = $('#_freeinvoice_api_force_invoice_number');
        if ($dropdown.length > 0)
            $dropdown.select2();
        if ($date_pickers.length > 0)
            $date_pickers.datepicker({
                dateFormat: 'dd/mm/yy',
                numberOfMonths: 1,
                showButtonPanel: true
            });
        if ($sectional_controller.length == 1) {
            $(document).on('change', $sectional_controller, function() {
                sectional_toggler($sectional_controller);
            });
            sectional_toggler($sectional_controller);
        }
        if ($invoice_number_controller.length == 1) {
            invoice_number_toggler($invoice_number_controller[0]);
            $invoice_number_controller.change(function() { invoice_number_toggler(this, true); });
        }
        if ($('#_freeinvoice_api_tax_representative_field').length == 1) {
            $(document).on('change', '#_freeinvoice_api_tax_representative_field', function() {
                tax_representative_check(this);
            });
            tax_representative_check('#_freeinvoice_api_tax_representative_field');
        }
        if ($('#_freeinvoice_api_customer_is_a_private').length == 1) {
            $(document).on('change', '#_freeinvoice_api_customer_is_a_private', function() {
                private_or_company_check(this);
            });
            private_or_company_check('#_freeinvoice_api_customer_is_a_private');
        }

    });

    function tax_representative_check(ctrl) {
        $('.freeinvoice-api-surname-name,#_freeinvoice_api_denomination_field').closest('tr').hide();
        if ($(ctrl).val() == 'denomination')
            $('#_freeinvoice_api_denomination_field').closest('tr').show();
        else
            $('.freeinvoice-api-surname-name').closest('tr').show();
    }

    function private_or_company_check(ctrl) {
        if ($(ctrl).val() == 'private')
            $('.freeinvoice-api-h4private').addClass('freeinvoice-api-hidden');
        else
            $('.freeinvoice-api-h4private').removeClass('freeinvoice-api-hidden');
    }

    function sectional_toggler($sectional_controller) {
        if ($sectional_controller.is(':checked'))
            $('.freeinvoice-api-sectional-field').closest('tr').show();
        else $('.freeinvoice-api-sectional-field').closest('tr').hide();
    }

    function invoice_number_toggler(el, animate) {
        var isChecked = el.checked,
            show = 'show',
            hide = 'hide';
        if (animate === true) {
            show = 'fadeIn';
            hide = 'fadeOut';
        }
        if (isChecked) {
            $('#_freeinvoice_api_invoice_number_auto')[hide]();
            $('#_freeinvoice_api_invoice_number_field')[show]();
        } else {
            $('#_freeinvoice_api_invoice_number_field')[hide]();
            $('#_freeinvoice_api_invoice_number_auto')[show]();
        }
    }

})(jQuery);

function send_freeinvoice_api_invoice(order_id, wpnonce) {
    console.log("invio in corso..." + order_id);
    jQuery('#ajax-send-invoice-button-' + order_id).hide();
    jQuery('#ajax-send-invoice-response-' + order_id).show();
    jQuery.ajax({
        type: "POST",
        url: ajaxurl,
        data: {
            action: 'send_order_invoice',
            order_id: order_id,
            _wpnonce: wpnonce
        }
    }).done(function(response) {
        jQuery('#ajax-send-invoice-response-' + order_id).html('<pre>' + response + '</pre>');
    });
}