<?php

/** 
 * FreeInvoice API
 *
 * This plugin allows you to transform the orders 
 * received in your online store made with WooCommerce, 
 * in italian electronic invoices via 
 * FreeInvoice API service (https://www.cloudfinance.it/free-invoice-api.html)
 *
 * @link              https://www.freeinvoice.it/
 * @since             1.0.0
 * @package           freeinvoice-api
 * 
 * @wordpress-plugin
 * Plugin Name:       FreeInvoice API
 * Plugin URI:        https://www.wordpress.org/plugins/freeinvoice-api/
 * Description:       Plugin di FreeInvoice per la fatturazione elettronica con WooCommerce.
 * Version:           1.0.0
 * Requires at least: 5.2
 * Requires PHP:      7.2
 * Author:            Cloud Finance srl
 * Author URI:        https://www.cloudfinance.it/
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       freeinvoice-api
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
	die;
}
/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define('FREEINVOICE_API_VERSION', '1.0.0');

if (!function_exists('is_plugin_active')) {
	include_once(ABSPATH . '/wp-admin/includes/plugin.php');
}

/**
 * Check for the existence of WooCommerce and any other requirements
 */
function freeinvoice_check_requirements()
{
	if (is_plugin_active('woocommerce/woocommerce.php')) {
		return true;
	} else {
		add_action('admin_notices', 'freeinvoice_missing_wc_notice');
		return false;
	}
}

/**
 * Display a message advising WooCommerce is required
 */
function freeinvoice_missing_wc_notice()
{
	$class = 'notice notice-error';
	$message = __('FreeInvoice API requires WooCommerce to be installed and actived.', 'freeinvoice-api');

	printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-freeinvoice-api-activator.php
 */
function activate_freeinvoice_api()
{
	require_once plugin_dir_path(__FILE__) . 'includes/class-freeinvoice-api-activator.php';
	Freeinvoice_Api_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-freeinvoice-api-deactivator.php
 */
function deactivate_freeinvoice_api()
{
	require_once plugin_dir_path(__FILE__) . 'includes/class-freeinvoice-api-deactivator.php';
	Freeinvoice_Api_Deactivator::deactivate();
}

add_action('plugins_loaded', 'freeinvoice_check_requirements');

register_activation_hook(__FILE__, 'activate_freeinvoice_api');
register_deactivation_hook(__FILE__, 'deactivate_freeinvoice_api');

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path(__FILE__) . 'includes/class-freeinvoice-api.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_freeinvoice_api()
{
	if (freeinvoice_check_requirements()) {
		$plugin = new Freeinvoice_Api();
		$plugin->run();
	}
}
run_freeinvoice_api();
