<?php

/**
 * Fired during plugin activation
 *
 * @link       https://www.cloudfinance.it/
 * @since      1.0.0
 *
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/includes
 * @author     Cloud Finance srl <supportemail@cloudfinance.it>
 */
class Freeinvoice_Api_Billing
{
    const DB_PREFIX = '_freeinvoice_api_';
    public static function get_order_id($invoiceId)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'freeinvoice_api_conn';
        return $wpdb->get_var($wpdb->prepare("SELECT `order_id` FROM `$table_name` WHERE `invoice_id` =%s", $invoiceId));
    }
    public static function get_ade_payment_methods()
    {
        return array(
            'MP01' => __('contanti', 'freeinvoice-api'),
            'MP02' => __('assegno', 'freeinvoice-api'),
            'MP03' => __('assegno circolare', 'freeinvoice-api'),
            'MP04' => __('contanti presso Tesoreria', 'freeinvoice-api'),
            'MP05' => __('bonifico', 'freeinvoice-api'),
            'MP06' => __('vaglia cambiario', 'freeinvoice-api'),
            'MP07' => __('bollettino bancario', 'freeinvoice-api'),
            'MP08' => __('carta di pagamento', 'freeinvoice-api'),
            'MP09' => __('RID', 'freeinvoice-api'),
            'MP10' => __('RID utenze', 'freeinvoice-api'),
            'MP11' => __('RID veloce', 'freeinvoice-api'),
            'MP12' => __('RIBA', 'freeinvoice-api'),
            'MP13' => __('MAV', 'freeinvoice-api'),
            'MP14' => __('quietanza erario', 'freeinvoice-api'),
            'MP15' => __('giroconto su conti di contabilità speciale', 'freeinvoice-api'),
            'MP16' => __('domiciliazione bancaria', 'freeinvoice-api'),
            'MP17' => __('domiciliazione postale', 'freeinvoice-api'),
            'MP18' => __('bollettino di c/c postale', 'freeinvoice-api'),
            'MP19' => __('SEPA Direct Debit', 'freeinvoice-api'),
            'MP20' => __('SEPA Direct Debit CORE', 'freeinvoice-api'),
            'MP21' => __('SEPA Direct Debit B2B', 'freeinvoice-api'),
            'MP22' => __('Trattenuta su somme già riscosse', 'freeinvoice-api'),
        );
    }
    private static $billing = array(
        '_freeinvoice_api_customer_is_a_private' => array(
            'class'     => array('form-row-wide'),
            'clear'     => true,
            'label'     => 'Persona fisica o Azienda/Professionista',
            'options' => array('private' => 'Persona fisica', 'company' => 'Azienda/Professionista'),
            'required'  => true,
            'type' => 'select',
            'priority' => 9
        ),
        '_freeinvoice_api_billing_tax_code' => array(
            'class'     => array('form-row-wide', 'freeinvoice-api-private-required'),
            'label'     => 'Codice Fiscale',
            'type' => 'text',
            'priority' => 93
        ),
        '_freeinvoice_api_billing_vat_code' => array(
            'class'     => array('form-row-wide', 'freeinvoice-api-h4private', 'freeinvoice-api-company-required'),
            'clear'     => true,
            'label'     => 'Partita IVA',
            'type' => 'text',
            'priority' => 92
        ),
        '_freeinvoice_api_sdi_code' => array(
            'class'     => array('form-row-wide', 'freeinvoice-api-h4private', 'freeinvoice-api-company-required'),
            'label'     => 'Codice Destinatario (SDI)',
            'type' => 'text',
            'priority' => 91
        ),
        '_freeinvoice_api_pec_destinatario' => array(
            'class'     => array('form-row-wide', 'freeinvoice-api-h4private'),
            'label'     => 'Indirizzo PEC',
            'type' => 'text',
            'priority' => 94
        ),
        '_freeinvoice_api_optional' => array(
            'class'     => array('form-row-wide', 'freeinvoice-api-hidden'),
            'label'     => 'Optional',
            'type' => 'text',
            'required'  => false,
        ),

    );
    public static function get_billing_data($order_id, &$errors)
    {
        $store_country = 'IT';
        $supplier_vat_code = get_option(self::DB_PREFIX . 'vatcode_field');
        $supplier_tax_code = get_option(self::DB_PREFIX . 'taxcode_field');
        if (get_option(self::DB_PREFIX . 'tax_representative_field', 'denomination') == 'denomination') {
            $supplier_name = $supplier_surname = null;
            $supplier_company = get_option(self::DB_PREFIX . 'denomination_field'); 
            $supplier_company =  trim(self::clean($supplier_company));
        } else {
            $supplier_company = null;
            $supplier_name = get_option(self::DB_PREFIX . 'name_field');
            $supplier_surname = get_option(self::DB_PREFIX . 'surname_field');
        }
        $supplier_store_address = get_option('woocommerce_store_address');
        $supplier_store_postcode = get_option('woocommerce_store_postcode');
        $supplier_store_city = get_option('woocommerce_store_city');
        $store_state   = self::get_store_state(); //provincia (sigla)
        $supplier_tax_regime = get_option(self::DB_PREFIX . 'taxregime_field');

        $weight_unit = get_option('woocommerce_weight_unit');

        /**order data */
        $invoice_number = self::get_invoice_number($order_id);


        $errors = array();
        $order = wc_get_order($order_id);
        $order_data = $order->get_data();

        $customer_is_a_private = self::get_costumer_is_a_private_field($order_id) === 'private';
        if (!self::private_invoice_enabled() && ($customer_is_a_private)) {
            $errors[] = __('fatturazione a privati disabilitata', 'freeinvoice-api'); //fatturazione a privati disabilitata
            return false; //non dovrebbe arrivare qui perché i controlli su fatturazione a privati abilitata vengo fatti anche a monte    
        }

        $invoice_date = self::get_billing_date($order_id); //YYYY-MM-DD
        if (empty($invoice_date) || preg_match('/\d{4}-\d{2}-\d{2}/', $invoice_date) !== 1)
            $errors['invoice_date'] = __('Data di fatturazione non valida', 'freeinvoice-api');

        $billing_country = self::get_billing_country($order_id);
        if (strlen($billing_country) != 2)
            $errors[] = __('Nazione cliente non valida per la fatturazione', 'freeinvoice-api');

        $sdi_code = self::get_sdi_code($order_id);
        if (strlen($sdi_code) != 7)
            $errors['sdi_code'] = __('Codice destinatario non valido', 'freeinvoice-api');
        else if ($billing_country == 'IT' && $sdi_code == 'XXXXXXX')
            $errors['sdi_code'] = __('codice destinatario non valido per nazione cliente italiana', 'freeinvoice-api', 'freeinvoice-api');
        else if ($billing_country != 'IT' && $sdi_code != 'XXXXXXX')
            $errors['sdi_code'] = __('Per le fatture "transfrontaliere" il codice destinatario deve essere valorizzato con tutte "X" (\'XXXXXXX\')', 'freeinvoice-api');

        $pec_destinatario = self::get_pec($order_id);
        if (isset($pec_destinatario) && !empty($pec_destinatario))
            if (!filter_var($pec_destinatario, FILTER_VALIDATE_EMAIL))
                $errors['pec_destinatario'] = __('Pec destinatario non valida', 'freeinvoice-api');

        $billing_vat_code = self::get_billing_vat($order_id, $billing_country);
        $billing_tax_code  = self::get_billing_tax($order_id, $billing_country);
        $billing_name = trim($order_data['billing']['first_name']);
        $billing_surname = trim($order_data['billing']['last_name']);
        $billing_company = trim($order_data['billing']['company']);

        if (!$customer_is_a_private) {
            if (empty($billing_vat_code))
                $errors['billing_vat_code'] = __('Partita iva del cliente obbligatoria', 'freeinvoice-api');
            else if (strlen($billing_vat_code) < 1 || strlen($billing_vat_code) > 28)
                $errors['billing_vat_code'] = __('Partita iva del cliente non valida', 'freeinvoice-api');
            if (!empty($billing_tax_code))
                if (($billing_vat_code != $billing_tax_code) && !Freeinvoice_Api_Validator::is_a_valid_tax_code($billing_tax_code))
                    $errors['billing_tax_code'] = __('codice fiscale non valido o non coerente con la partita iva', 'freeinvoice-api');
            if (empty($billing_company) && (empty($billing_name) || empty($billing_surname)))
                $errors[] = __('compilare il campo denominazione oppure nome e cognome del cliente nei dati di fatturazione di woocommerce', 'freeinvoice-api');
        } else {
            if (empty($billing_tax_code) || !Freeinvoice_Api_Validator::is_a_valid_tax_code($billing_tax_code))
                $errors['billing_tax_code'] = __('codice fiscale obbligatorio non valido', 'freeinvoice-api');
            if (empty($billing_name) || empty($billing_surname))
                $errors[] = __('compilare i campi nome e cognome del cliente nei dati di fatturazione di woocommerce', 'freeinvoice-api');
        }

        $payment_method = self::get_payment_method($order_id);

        $billing_address  = $order_data['billing']['address_1'];
        $billing_postcode  = $order_data['billing']['postcode'];
        $billing_city = $order_data['billing']['city'];
        $billing_state = $order_data['billing']['state'];

        $order_currency = $order_data['currency'];


        $order_total = $order->get_total(); //0.00
        
        $shipping_address  = self::get_shipping_or_billing($order_data, 'address_1'); //se non settati == billing
        $shipping_postcode  = self::get_shipping_or_billing($order_data, 'postcode');
        $shipping_city = self::get_shipping_or_billing($order_data, 'city');
        $shipping_country = strtoupper(self::get_shipping_or_billing($order_data, 'country'));

        $shipping_name =  self::get_shipping_or_billing($order_data, 'first_name');
        $shipping_surname = self::get_shipping_or_billing($order_data, 'last_name');
        $shipping_company = self::get_shipping_or_billing($order_data, 'company');
        $shipping_state = strtoupper(self::get_shipping_or_billing($order_data, 'state'));

        return array(
            'customer_is_a_private' => $customer_is_a_private,
            'sdi_code' => $sdi_code,
            'pec_destinatario' => $pec_destinatario,
            'store_country' => $store_country,
            'supplier_vat_code' => $supplier_vat_code,
            'supplier_tax_code' => $supplier_tax_code,
            'supplier_company' => $supplier_company,
            'supplier_name' => $supplier_name,
            'supplier_surname' => $supplier_surname,
            'supplier_tax_regime' => $supplier_tax_regime,
            'supplier_store_address' => $supplier_store_address,
            'supplier_store_postcode' => $supplier_store_postcode,
            'supplier_store_city' => $supplier_store_city,
            'store_state' => $store_state,
            'billing_country' => $billing_country,
            'billing_vat_code' => $billing_vat_code,
            'billing_tax_code' => $billing_tax_code,
            'billing_name' => $billing_name,
            'billing_surname' => $billing_surname,
            'billing_company' => $billing_company,
            'billing_address' => $billing_address,
            'billing_postcode' => $billing_postcode,
            'billing_city' => $billing_city,
            'billing_state' => $billing_state,
            'order_currency' => $order_currency,
            'invoice_date' => $invoice_date,
            'invoice_number' => $invoice_number,
            'order_total' => $order_total,
            'weight_unit' => $weight_unit,
            //'total_weight' => $total_weight,
            'shipping_address' => $shipping_address,
            'shipping_postcode' => $shipping_postcode,
            'shipping_city' => $shipping_city,
            'shipping_country' => $shipping_country,
            'shipping_name' => $shipping_name,
            'shipping_surname' => $shipping_surname,
            'shipping_company' => $shipping_company,
            'shipping_state' => $shipping_state,
            'payment_method' => $payment_method,
        );
    }
    public static function get_billing_fields_array()
    {
        $billing = self::translated_billing();
        if (!self::private_invoice_enabled())
            $billing[self::DB_PREFIX . 'billing_tax_code']['class'][] = 'freeinvoice-api-h4private';

        return $billing;
    }
    public static function get_billing_fields_for_db()
    {
        $billing = [];
        foreach (self::$billing as $name => $field)
            $billing[$name] = $field['label'];
        return $billing;
    }
    private static function get_billing_date($order_id)
    {
        $billing_date = self::format_date_for_invoice(get_post_meta($order_id, self::DB_PREFIX . 'invoice_date', true));
        if (!isset($billing_date) || preg_match('/\d{4}-\d{2}-\d{2}/', $billing_date) !== 1) {
            $order = wc_get_order($order_id);
            $completed_date = $order->get_date_completed();
            if (isset($completed_date)) {
                $billing_date = $completed_date->format('d/m/Y');
                Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'invoice_date', $billing_date);
                return self::format_date_for_invoice($billing_date);
            }
        }
        return $billing_date;
    }
    public static function update_billing_fields_for_admin_order($post_id, $post_data)
    {
        $order = wc_get_order($post_id);
        $field_names = array_keys(self::get_billing_fields_for_admin_order($order));
        foreach ($field_names as $id) {
            if(isset($post_data[$id]))
            Freeinvoice_Api_Validator::update_order_meta($post_id, $id, $post_data[$id]);
        }
    }
    public static function get_billing_fields_for_admin_order($order)
    {
        $billing = self::get_billing_fields_array();
        $order_id = $order->get_id();
        $completed_date = $order->get_date_completed();
        if (isset($completed_date))
            $completed_date = $completed_date->format('d/m/Y');

        $methods = self::get_ade_payment_methods();

        $admin_fields = array(
            self::DB_PREFIX . 'invoice_date' => array(
                'class'     => array('form-row-wide,'),
                'input_class' => array('freeinvoice-api-datepicker'),
                'label'     => __('Data Fattura', 'freeinvoice-api'),
                'type' => 'text',
                'value' => $completed_date,
            ),
            self::DB_PREFIX . 'force_invoice_number' => array(
                'class'     => array('form-row-wide'),
                'clear'     => true,
                'label'     => 'Forza numero fattura',
                'type' => 'checkbox',
                'required'  => false,
            ),
            self::DB_PREFIX . 'invoice_number' => array(
                'class'     => array('form-row-wide,'),
                'label'     => __('Numero Fattura', 'freeinvoice-api'),
                'type' => 'text',
                'disabled' => true
            ),
            self::DB_PREFIX . 'payment_method' => array(
                'class'     => array('form-row-wide,'),
                'label'     => __('Metodo di pagamento rilevato nell ordine', 'freeinvoice-api'),
                'type' => 'select',
                'options' => $methods,
                'value' => '',
            )
        );
        if (!self::private_invoice_enabled())
            foreach ($admin_fields as &$field)
                $field['class'][] = 'freeinvoice-api-h4private';

        $billing += $admin_fields;
        $val = null;
        foreach ($billing as $name => &$field) {
            $val = get_post_meta($order_id, $name, true);
            if (!empty($val))
                $field['value'] = $val;
        }
        return $billing;
    }
    private static function translated_billing()
    {
        $billing = self::$billing;
        foreach ($billing as &$field) {
            if (isset($field['label']))
                $field['label'] = __($field['label'], 'freeinvoice-api');

            if (isset($field['options']))
                foreach ($field['options'] as &$opt_label)
                    $opt_label = __($opt_label, 'freeinvoice-api');
        }
        return $billing;
    }
    public static function private_invoice_enabled()
    {
        return filter_var(get_option(self::DB_PREFIX . 'private_enabled', 'yes'), FILTER_VALIDATE_BOOLEAN);
    }
    public static function is_sandbox_mode()
    {
        return filter_var(get_option(self::DB_PREFIX . 'sandbox_mode', true), FILTER_VALIDATE_BOOLEAN);
    }
    /****** order meta data computed */
    private static function get_billing_country($order_id)
    {
        $order = wc_get_order($order_id);
        $order_data = $order->get_data();
        return strtoupper(trim(self::clean($order_data['billing']['country'])));
    }
    private static function get_costumer_is_a_private_field($order_id)
    {
        $stored = get_post_meta($order_id, self::DB_PREFIX . 'customer_is_a_private', true);
        $cpriv =  strtolower(trim(self::clean($stored)));
        if (!isset($cpriv) || empty($cpriv))
            $cpriv = 'private';
        if ($stored !== $cpriv)
            Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'customer_is_a_private', $cpriv);
        return $cpriv;
    }
    private static function get_sdi_code($order_id)
    {
        $stored_sdi = get_post_meta($order_id, self::DB_PREFIX . 'sdi_code', true);
        $sdi =  strtoupper(trim(self::clean($stored_sdi)));
        if (!isset($sdi) || empty($sdi))
            $sdi = '0000000';
        if ($stored_sdi !== $sdi)
            Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'sdi_code', $sdi);
        return $sdi;
    }
    private static function get_pec($order_id)
    {
        $stored_pec =  get_post_meta($order_id, self::DB_PREFIX . 'pec_destinatario', true);
        $pec = strtolower(trim($stored_pec));
        if ($stored_pec !== $pec)
            Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'pec_destinatario', $pec);
        return $pec;
    }
    private static function get_billing_vat($order_id, $billing_country)
    {
        $stored_vat = get_post_meta($order_id, self::DB_PREFIX . 'billing_vat_code', true);
        $vat = trim(self::clean($stored_vat));
        if ($billing_country == 'IT')
            $vat = Freeinvoice_Api_Validator::validate_italian_vat_code($vat);
        if ($stored_vat !== $vat)
            Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'billing_vat_code', $vat);

        return $vat;
    }
    private static function get_billing_tax($order_id)
    {
        $stored_tax = get_post_meta($order_id, self::DB_PREFIX . 'billing_tax_code', true);
        $tax = strtoupper(trim(self::clean($stored_tax)));
        $tax = Freeinvoice_Api_Validator::validate_italian_vat_code($tax);
        if ($stored_tax !== $tax)
            Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'billing_tax_code', $tax);
        return $tax;
    }
    private static function get_payment_method($order_id)
    {
        $stored_payment = get_post_meta($order_id, self::DB_PREFIX . 'payment_method', true);
        $payment = strtoupper(trim(self::clean($stored_payment)));
        if (empty($payment))
            $payment = self::match_payment_by_order($order_id);
        if ($stored_payment !== $payment)
            Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'payment_method', $payment);
        return $payment;
    }
    private static function match_payment_by_order($order_id)
    {

        $key = get_post_meta($order_id, '_payment_method', true);
        $stored = self::get_aviable_wc_payments();
        $ade_arr = self::get_ade_payment_methods();

        $value = &$stored[$key];
        if (!empty($value) && isset($ade_arr[$value['value']]))
            return $value['value'];
        else
            return array_keys($ade_arr)[0];
    }


    private static function get_store_country()
    {
        return 'IT';
    }

    private static function get_store_state()
    {
        if (self::get_store_country() === 'IT') {
            $store_raw_country = esc_attr(get_option('woocommerce_default_country'));
            $split_country = explode(":", $store_raw_country);
            $store_state = strtoupper($split_country[1]);
            if (strlen($store_state) === 2) return $store_state;
            else throw new Exception("Error FreeInvoice API controllare informazioni Paese del negozio woocommerce", 1);
        } else throw new Exception("Error FreeInvoice API il negozio woocommerce deve essere italiano", 1);
        return null;
    }
    //converte la data dal formato dd/mm/yyyy a yyyy-mm-dd
    private static function format_date_for_invoice($date)
    {
        if (strpos($date, '/') == 2) {
            $dt = explode('/', $date);
            return $dt[2] . '-' . $dt[1] . '-' . $dt[0];
        } else return null;
    }
    private static function get_shipping_or_billing($order_data, $field)
    {
        if (!empty($order_data['shipping'][$field]))
            return $order_data['shipping'][$field];
        else return $order_data['billing'][$field];
    }
    private static function is_invoice_number_forced($order_id = null){
      return isset($order_id) && filter_var(get_post_meta($order_id, self::DB_PREFIX . 'force_invoice_number', true), FILTER_VALIDATE_BOOLEAN);
    }
    public static function get_invoice_number($order_id = null)
    {
        $nr = null;
        if (self::is_invoice_number_forced())
            $nr =  get_post_meta($order_id, self::DB_PREFIX . 'invoice_number', true);
        if (isset($nr) && !empty(trim($nr))) return trim($nr);
        else {
            $nr = intval(get_option(self::DB_PREFIX . 'next_invoice_field', 1));
            if ($nr <= 0) $nr = 1;
            if (filter_var(get_option(self::DB_PREFIX . 'sectional_enabled', 'no'), FILTER_VALIDATE_BOOLEAN)) {
                $prefix = get_option(self::DB_PREFIX . 'sectional_start_field', '');
                $suffix = get_option(self::DB_PREFIX . 'sectional_end_field', '');
            } else $prefix = $suffix = '';
            $nr = trim($prefix . $nr . $suffix);
            if (isset($order_id))
                Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'invoice_number', $nr);
            return $nr;
        }
    }
    public static function is_send_invoice_aviable($order_id, &$errors)
    {
        $errors = array();
        if (!empty(Freeinvoice_Api_Validator::config_errors()))
            $errors[] = __('Controlla la configurazione del plugin di FreeInvoice API', 'freeinvoice-api');
        $customer_is_a_private = self::get_costumer_is_a_private_field($order_id) === 'private';
        if ($customer_is_a_private)
            if (!self::private_invoice_enabled())
                $errors[] = __('fatturazione a privati disabilitata', 'freeinvoice-api');
            else if (self::get_billing_country($order_id) !== 'IT')
                $errors[] = __('fatturazione elettronica verso clienti privati esteri non consentita', 'freeinvoice-api');
        $order = wc_get_order($order_id);
        $order_data = $order->get_data();
        $order_status = $order_data['status'];
        if (!($order_status == 'completed' || $order_status == 'wc-completed'))
            $errors[] = __('l\'ordine non risulta ancora completato', 'freeinvoice-api'); 
        return empty($errors);
    }
    public static function get_aviable_zero_taxes()
    {
        $all_tax_rates = [];
        $tax_classes = WC_Tax::get_tax_class_slugs(); // Retrieve all tax classes.
        $names = WC_Tax::get_tax_classes(); // Retrieve all tax classes names.
        foreach ($tax_classes as $index => $slug) { // For each tax class, get all rates.
            $taxes = WC_Tax::get_rates_for_tax_class($slug);
            $total_rate = WC_Tax::get_tax_total(array_column($taxes, 'tax_rate'));
            if ($total_rate === 0.0)
                $all_tax_rates[$slug] = $names[$index];
        }
        return $all_tax_rates;
    }
    public static function update_natura_taxes($post)
    {
        $stored = get_option(self::DB_PREFIX . 'tax_natura', array());
        $tax_classes = self::get_aviable_zero_taxes();
        foreach ($tax_classes as $slug => $name)
            if (isset($post[self::DB_PREFIX . $slug]))
                $stored[$slug] = $post[self::DB_PREFIX . $slug];
        update_option(self::DB_PREFIX . 'tax_natura', $stored);
    }
    public static function get_tax_natura($tax_class_slug)
    {
        $stored = get_option(self::DB_PREFIX . 'tax_natura', array());
        $tax_classes = self::get_aviable_zero_taxes();
        if (isset($tax_classes[$tax_class_slug]))
            if (isset($stored[$tax_class_slug]))
                return $stored[$tax_class_slug];
            else return 'N1';
        return null;
    }
    public static function get_aviable_wc_payments()
    {
        $gateways = WC()->payment_gateways->get_available_payment_gateways();
        $stored = get_option(self::DB_PREFIX . 'payment_methods', array());
        $ade_arr = self::get_ade_payment_methods();

        $enabled_gateways = [];

        if ($gateways)
            foreach ($gateways as $id => $gateway)
                if ($gateway->enabled == 'yes') {
                    $value = &$stored[$id];
                    if (empty($value) || !isset($ade_arr[$value]))
                        $value = array_values($ade_arr)[0];
                    $enabled_gateways[$id] = array(
                        'title' => $gateway->get_method_title(),
                        'value' => $value
                    );
                }

        return $enabled_gateways;
    }
    public static function update_payment_config($post)
    {
        $stored = get_option(self::DB_PREFIX . 'payment_methods', array());
        $new = $stored;
        $ade_arr = self::get_ade_payment_methods();
        $methods = self::get_aviable_wc_payments();
        foreach ($methods as $id => $field) {
            $ade_id = &$post[self::DB_PREFIX . $id];
            if (
                isset($ade_id) &&
                isset($ade_arr[$ade_id]) &&
                (!isset($stored[$id]) || $ade_id !== $stored[$id])
            )
                $new[$id] = $ade_id;
        }
        if ($new != $stored)
            update_option(self::DB_PREFIX . 'payment_methods', $new);
    }
    public static function get_invoice_status($order_id)
    {
        return intval(get_post_meta($order_id, self::DB_PREFIX . 'invoice_status', true));
    }
    public static function update_invoice_status($order_id, $new_status)
    {
        $invoice_status = intval($new_status);
        if ($invoice_status >= 4 && $invoice_status <= 9)
            Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'invoice_status', $invoice_status);
    }
    public static function set_invoice_id($order_id, $invoiceId)
    {
        global $wpdb;
        $table = $wpdb->prefix . 'freeinvoice_api_conn';
        $wpdb->insert($table, array('invoice_id' => $invoiceId, 'order_id' => $order_id), array('%s', '%d'));

        Freeinvoice_Api_Validator::update_order_meta($order_id, self::DB_PREFIX . 'invoice_id', $invoiceId);
    }
    public static function update_next_invoice_number($order_id)
    { if(!self::is_invoice_number_forced($order_id)){
        $nr = intval(get_option(self::DB_PREFIX . 'next_invoice_field', 1));
        update_option(self::DB_PREFIX . 'next_invoice_field', $nr + 1);
    }
    }
    private static function clean($string)
    {
        return preg_replace('/[^A-Za-z0-9\-]/', ' ', $string); // Removes special chars.
    }
}
