<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.cloudfinance.it/
 * @since      1.0.0
 *
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/public
 * @author     Cloud Finance srl <supportemail@cloudfinance.it>
 */
class Freeinvoice_Api_Public
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Freeinvoice_Api_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Freeinvoice_Api_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/freeinvoice-api-public.css', array(), $this->version, 'all');
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Freeinvoice_Api_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Freeinvoice_Api_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/freeinvoice-api-public.js', array('jquery'), $this->version, false);
	}
	/**
	 * Definizione dei campi di checkout personalizzati
	 *
	 * @param [type] $fields
	 * @return void
	 */

	public function add_freeinvoice_checkout_fields($fields)
	{
		$fields['billing'] += Freeinvoice_Api_Billing::get_billing_fields_array();
		return $fields;
	}
	/**
	 * Update the order meta with field value
	 */
	public function store_freeinvoice_checkout_fields($order_id)
	{
		$billing = Freeinvoice_Api_Billing::get_billing_fields_for_db();
		foreach ($billing as $name => $label)
			if (!empty($_POST[$name]))
				Freeinvoice_Api_Validator::update_order_meta($order_id, $name, $_POST[$name]);
	}
	public function custom_webhook_endpoint()
	{
		if (isset($_REQUEST['webhook']) && $_REQUEST['webhook'] === 'freeinvoice-api') {
			Freeinvoice_Api_Comm::webhook_notification();
		}
	}
}
